(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@yellicode/elements'), require('@yellicode/templating')) :
    typeof define === 'function' && define.amd ? define(['exports', '@yellicode/elements', '@yellicode/templating'], factory) :
    (factory((global['@yellicode/csharp'] = {}),global.elements,global.templating));
}(this, (function (exports,elements,templating) { 'use strict';

    /**
     * Enumerates the possible collection types to generated for properties and parameters.
     */
    (function (CollectionType) {
        CollectionType[CollectionType["ICollection"] = 0] = "ICollection";
        CollectionType[CollectionType["IEnumerable"] = 1] = "IEnumerable";
        CollectionType[CollectionType["IList"] = 2] = "IList";
    })(exports.CollectionType || (exports.CollectionType = {}));
    (function (NamespaceFeatures) {
        NamespaceFeatures[NamespaceFeatures["None"] = 0] = "None";
        NamespaceFeatures[NamespaceFeatures["All"] = 0] = "All";
    })(exports.NamespaceFeatures || (exports.NamespaceFeatures = {}));
    (function (ClassFeatures) {
        ClassFeatures[ClassFeatures["None"] = 0] = "None";
        ClassFeatures[ClassFeatures["XmlDocSummary"] = 1] = "XmlDocSummary";
        ClassFeatures[ClassFeatures["Generalizations"] = 2] = "Generalizations";
        ClassFeatures[ClassFeatures["InterfaceRealizations"] = 4] = "InterfaceRealizations";
        ClassFeatures[ClassFeatures["All"] = 7] = "All";
        ClassFeatures[ClassFeatures["AllExceptXmlDocs"] = 6] = "AllExceptXmlDocs";
    })(exports.ClassFeatures || (exports.ClassFeatures = {}));
    (function (InterfaceFeatures) {
        InterfaceFeatures[InterfaceFeatures["None"] = 0] = "None";
        InterfaceFeatures[InterfaceFeatures["XmlDocSummary"] = 1] = "XmlDocSummary";
        InterfaceFeatures[InterfaceFeatures["Generalizations"] = 2] = "Generalizations";
        InterfaceFeatures[InterfaceFeatures["All"] = 3] = "All";
        InterfaceFeatures[InterfaceFeatures["AllExceptXmlDocs"] = 2] = "AllExceptXmlDocs";
    })(exports.InterfaceFeatures || (exports.InterfaceFeatures = {}));
    (function (EnumFeatures) {
        EnumFeatures[EnumFeatures["None"] = 0] = "None";
        EnumFeatures[EnumFeatures["XmlDocSummary"] = 1] = "XmlDocSummary";
        /**
         * Writes enumeration member initializers. This flag only applies when values are provided in the model.
         */
        EnumFeatures[EnumFeatures["Initializers"] = 2] = "Initializers";
        EnumFeatures[EnumFeatures["All"] = 3] = "All";
        EnumFeatures[EnumFeatures["AllExceptXmlDocs"] = 2] = "AllExceptXmlDocs";
    })(exports.EnumFeatures || (exports.EnumFeatures = {}));
    (function (EnumMemberFeatures) {
        EnumMemberFeatures[EnumMemberFeatures["None"] = 0] = "None";
        EnumMemberFeatures[EnumMemberFeatures["XmlDocSummary"] = 1] = "XmlDocSummary";
        /**
         * Writes enumeration member initializers. This flag only applies when values are provided in the model.
         */
        EnumMemberFeatures[EnumMemberFeatures["Initializers"] = 2] = "Initializers";
        EnumMemberFeatures[EnumMemberFeatures["All"] = 3] = "All";
        EnumMemberFeatures[EnumMemberFeatures["AllExceptXmlDocs"] = 2] = "AllExceptXmlDocs";
    })(exports.EnumMemberFeatures || (exports.EnumMemberFeatures = {}));
    (function (PropertyFeatures) {
        PropertyFeatures[PropertyFeatures["None"] = 0] = "None";
        PropertyFeatures[PropertyFeatures["XmlDocSummary"] = 1] = "XmlDocSummary";
        /**
         * The access modifier if the owner is not an Interface.
         */
        PropertyFeatures[PropertyFeatures["AccessModifier"] = 2] = "AccessModifier";
        PropertyFeatures[PropertyFeatures["OptionalModifier"] = 4] = "OptionalModifier";
        PropertyFeatures[PropertyFeatures["All"] = 7] = "All";
        PropertyFeatures[PropertyFeatures["AllExceptXmlDocs"] = 6] = "AllExceptXmlDocs";
    })(exports.PropertyFeatures || (exports.PropertyFeatures = {}));
    (function (MethodFeatures) {
        MethodFeatures[MethodFeatures["None"] = 0] = "None";
        MethodFeatures[MethodFeatures["XmlDocSummary"] = 1] = "XmlDocSummary";
        MethodFeatures[MethodFeatures["XmlDocParameters"] = 2] = "XmlDocParameters";
        MethodFeatures[MethodFeatures["XmlDocReturns"] = 4] = "XmlDocReturns";
        /**
        * The access modifier if the owner is not an Interface.
        */
        MethodFeatures[MethodFeatures["AccessModifier"] = 8] = "AccessModifier";
        MethodFeatures[MethodFeatures["All"] = 15] = "All";
        MethodFeatures[MethodFeatures["AllExceptXmlDocs"] = 8] = "AllExceptXmlDocs";
    })(exports.MethodFeatures || (exports.MethodFeatures = {}));

    class CSharpTypeNameProvider extends templating.DefaultTypeNameProvider {
        getDataTypeName(typedElement) {
            if (!typedElement || !typedElement.type)
                return null;
            const t = typedElement.type;
            if (elements.isPrimitiveBoolean(t))
                return "bool";
            if (elements.isPrimitiveInteger(t))
                return "int";
            if (elements.isPrimitiveReal(t))
                return "double"; // By default, a real numeric literal on the right side of the assignment operator is treated as double (https://msdn.microsoft.com/en-us/library/b1e65aza.aspx)
            if (elements.isPrimitiveString(t))
                return "string";
            if (elements.isPrimitiveObject(t))
                return "object";
            return super.getDataTypeName(typedElement);
        }
        static canBeNullable(type) {
            if (!type || type.name == null || elements.isPrimitiveString(type) || elements.isPrimitiveObject(type))
                return false; // type has no name or is already nullable
            return elements.isEnumeration(type) || elements.isDataType(type); // isDataType includes PrimitiveType      
        }
    }

    class CSharpCommentWriter {
        constructor(writer, typeNameProvider, maxCommentWidth) {
            this.writer = writer;
            this.typeNameProvider = typeNameProvider;
            this.maxCommentWidth = maxCommentWidth;
        }
        writeDelimitedCommentParagraph(text) {
            if (text == null)
                return;
            const lines = [text];
            this.writer.writeLine("/*");
            this.writeCommentLines(lines, "* ");
            this.writer.writeLine("*/");
        }
        writeDelimitedCommentLines(lines) {
            if (lines == null)
                return;
            this.writer.writeLine("/*");
            this.writeCommentLines(lines, "* ");
            this.writer.writeLine("*/");
        }
        writeCommentLines(lines, prefix) {
            if (!lines)
                return;
            lines.forEach(line => {
                if (line == null)
                    return;
                if (this.maxCommentWidth > 0 && line.length > this.maxCommentWidth) {
                    // See if we can split the line
                    const split = templating.CodeWriterUtility.wordWrap(line, this.maxCommentWidth);
                    split.forEach(s => {
                        this.writer.writeLine(`${prefix}${s}`);
                    });
                }
                else
                    this.writer.writeLine(`${prefix}${line}`);
            });
        }
    }

    class XmlDocUtility {
        static getXmlDocLineForParameter(parameter) {
            const commentBodies = XmlDocUtility.joinCommentBodies(parameter);
            return parameter.direction === elements.ParameterDirectionKind.return ? `<returns>${commentBodies}</returns>`
                : `<param name=\"${parameter.name}\">${commentBodies}</param>`;
        }
        static getXmlDocLinesForParameters(parameters) {
            if (parameters == null)
                return [];
            var lines = [];
            parameters.forEach((p) => {
                lines.push(XmlDocUtility.getXmlDocLineForParameter(p));
            });
            return lines;
        }
        static joinCommentBodies(element) {
            if (!element.ownedComments)
                return '';
            const bodies = element.ownedComments.map((c) => c.body);
            return bodies.join(' ');
        }
    }

    class CSharpWriter extends templating.CodeWriter {
        /**
         * Constructor. Creates a new CSharpWriter instance using the TextWriter and options provided.
         * @param writer The template's current TextWriter.
         * @param options Optional: the global options for this writer.
         */
        constructor(writer, options) {
            super(writer);
            if (!options)
                options = {};
            this.typeNameProvider = options.typeNameProvider || new CSharpTypeNameProvider();
            this.commentWriter = new CSharpCommentWriter(writer, this.typeNameProvider, options.maxCommentWidth || 100);
        }
        /**
         * Writes 1 or more using directives, each on a new line.
         * @param values A collection of strings, typically namespace names.
         */
        writeUsingDirectives(...values) {
            values.forEach(v => {
                this.writeLine(`using ${v};`);
            });
        }
        /**
         * Writes an indented block of code, wrapped in opening and closing brackets.
         * @param contents A callback function that writes the contents.
         */
        writeCodeBlock(contents) {
            this.writeLine('{');
            this.increaseIndent();
            if (contents)
                contents(this);
            this.decreaseIndent();
            this.writeLine('}');
        }
        ;
        writeNamespaceBlock(data, contents, options) {
            if (!data)
                return;
            if (!options)
                options = {};
            const features = (options.features === undefined) ? exports.NamespaceFeatures.All : options.features;
            if (typeof data === 'string') {
                this.writeLine(`namespace ${data}`);
            }
            else if (elements.isPackage(data)) {
                if (options.writeFullName) {
                    const allPackages = data.getNestingPackages();
                    allPackages.push(data);
                    this.writeLine(`namespace ${allPackages.map(p => p.name).join('.')}`);
                }
                else
                    this.writeLine(`namespace ${data.name}`);
            }
            this.writeCodeBlock(contents);
        }
        ;
        /**
         * Writes a block of code, wrapped in a class declaration and opening and closing brackets.
         * This function does not write class members.
         * @param cls The class.
         * @param contents A callback function that writes the class contents.
         * @param options An optional ClassOptions object.
         */
        writeClassBlock(cls, contents, options) {
            if (!cls)
                return;
            if (!options)
                options = {};
            const features = (options.features === undefined) ? exports.ClassFeatures.All : options.features;
            if (features & exports.ClassFeatures.XmlDocSummary) {
                this.writeXmlDocSummary(cls.ownedComments);
            }
            this.writeIndent();
            this.writeAccessModifier(cls.visibility);
            if (cls.isAbstract) {
                this.write('abstract ');
            }
            if (options.isPartial) {
                this.write('partial ');
            }
            this.write(`class ${cls.name}`);
            let hasGeneralizations = false;
            if (features & exports.ClassFeatures.Generalizations) {
                hasGeneralizations = this.writeGeneralizations(cls.generalizations, options.inherits);
            }
            if (features & exports.ClassFeatures.InterfaceRealizations) {
                this.writeInterfaceRealizations(cls.interfaceRealizations, options.implements, hasGeneralizations);
            }
            this.writeEndOfLine();
            this.writeCodeBlock(contents);
        }
        /**
        * Writes a block of code, wrapped in an interface declaration and opening and closing brackets.
        * This function does not write interface members.
        * @param iface The interface.
        * @param contents A callback function that writes the interface contents.
        * @param options An optional InterfaceOptions object.
        */
        writeInterfaceBlock(iface, contents, options) {
            if (!iface)
                return;
            if (!options)
                options = {};
            const features = (options.features === undefined) ? exports.InterfaceFeatures.All : options.features;
            if (features & exports.InterfaceFeatures.XmlDocSummary) {
                this.writeXmlDocSummary(iface.ownedComments);
            }
            this.writeIndent();
            this.writeAccessModifier(iface.visibility);
            if (options.isPartial) {
                this.write('partial ');
            }
            this.write(`interface ${iface.name}`);
            if (features & exports.InterfaceFeatures.Generalizations) {
                this.writeGeneralizations(iface.generalizations, options.inherits);
            }
            this.writeEndOfLine();
            this.writeCodeBlock(contents);
        }
        /**
        * Writes a block of code, wrapped in an enum declaration and opening and closing brackets.
        * This function does not write enumeration members. Use the writeEnumMember function
        * to write each individual member or the writeEnumeration function to write the full enumeration.
        * @param element The enumeration.
        * @param contents A callback function that writes the enumeration contents.
        * @param options An optional EnumerationOptions object.
        */
        writeEnumerationBlock(enumeration, contents, options) {
            if (!enumeration)
                return;
            if (!options)
                options = {};
            const features = (options.features === undefined) ? exports.EnumFeatures.All : options.features;
            if (features & exports.EnumFeatures.XmlDocSummary) {
                this.writeXmlDocSummary(enumeration.ownedComments);
            }
            this.writeIndent();
            this.writeAccessModifier(enumeration.visibility);
            this.write(`enum ${enumeration.name}`);
            this.writeEndOfLine();
            this.writeCodeBlock(contents);
        }
        /**
        * Writes a full enumeration, including members.
        * @param element The enumeration.
        * @param options An optional EnumOptions object.
        */
        writeEnumeration(enumeration, options) {
            if (!enumeration)
                return;
            if (!options)
                options = {};
            const features = (options.features === undefined) ? exports.EnumFeatures.All : options.features;
            // Pass on enum features to enum member features
            let enumMemberFeatures = exports.EnumMemberFeatures.None;
            if (features & exports.EnumFeatures.XmlDocSummary)
                enumMemberFeatures |= exports.EnumMemberFeatures.XmlDocSummary;
            if (features & exports.EnumFeatures.Initializers)
                enumMemberFeatures |= exports.EnumMemberFeatures.Initializers;
            this.writeEnumerationBlock(enumeration, () => {
                enumeration.ownedLiterals.forEach((literal) => {
                    this.writeEnumMember(literal, { features: enumMemberFeatures });
                });
            }, options);
        }
        /**
         * Writes an individual enumeration member.
         * @param literal The EnumerationLiteral for which to write the member.
         * @param options An optional EnumMemberOptions object.
         */
        writeEnumMember(literal, options) {
            if (!literal)
                return;
            if (!options)
                options = {};
            const features = (options.features === undefined) ? exports.EnumMemberFeatures.All : options.features;
            const isLast = literal.enumeration.ownedLiterals.indexOf(literal) === literal.enumeration.ownedLiterals.length - 1;
            if (features & exports.EnumMemberFeatures.XmlDocSummary) {
                this.writeXmlDocSummary(literal.ownedComments);
            }
            this.writeIndent();
            this.write(literal.name);
            if ((features & exports.EnumMemberFeatures.Initializers) && literal.specification) {
                let initialValue = literal.specification.getStringValue();
                this.write(` = ${initialValue}`);
            }
            if (!isLast) {
                this.write(',');
            }
            this.writeEndOfLine();
        }
        /**
        * Writes a method declaration without a body.
        * @param operation The operation for which to write the method.
        * @param options An optional MethodOptions object.
        */
        writeInterfaceMethod(operation, options) {
            if (!operation)
                return;
            if (!options)
                options = {};
            const features = (options.features === undefined) ? exports.MethodFeatures.All : options.features;
            if (features & exports.MethodFeatures.XmlDocSummary) {
                this.writeXmlDocSummary(operation.ownedComments);
            }
            if (features & exports.MethodFeatures.XmlDocParameters) {
                this.writeXmlDocParameters(operation);
            }
            if (features & exports.MethodFeatures.XmlDocReturns) {
                this.writeXmlDocReturns(operation);
            }
            this.writeIndent();
            // Write the return type
            this.write(`${this.getMethodReturnType(operation, options)} `);
            this.write(`${operation.name}(`);
            this.writeInOutParameters(operation.ownedParameters, options);
            this.write(');');
            this.writeEndOfLine();
        }
        /**
        * Writes an indented block of code, wrapped in a method declaration and opening and closing brackets.
        * @param operation The operation for which to write the method.
        * @param contents A callback function that writes the operation contents.
        * @param options An optional MethodOptions object.
        */
        writeClassMethodBlock(operation, contents, options) {
            if (!operation)
                return;
            if (!options)
                options = {};
            const features = (options.features === undefined) ? exports.MethodFeatures.All : options.features;
            if (features & exports.MethodFeatures.XmlDocSummary) {
                this.writeXmlDocSummary(operation.ownedComments);
            }
            if (features & exports.MethodFeatures.XmlDocParameters) {
                this.writeXmlDocParameters(operation);
            }
            if (features & exports.MethodFeatures.XmlDocReturns) {
                this.writeXmlDocReturns(operation);
            }
            this.writeIndent();
            this.writeAccessModifier(operation.visibility);
            if (operation.isStatic) {
                this.write('static ');
            }
            else if (operation.isAbstract) {
                this.write('abstract ');
            }
            else if (options.virtual) {
                this.write('virtual ');
            }
            this.write(`${this.getMethodReturnType(operation, options)} `);
            this.write(`${operation.name}(`);
            this.writeInOutParameters(operation.ownedParameters, options);
            this.write(')');
            if (operation.isAbstract) {
                this.write(';');
                this.writeEndOfLine();
            }
            else {
                this.writeEndOfLine();
                this.writeCodeBlock(contents);
            }
        }
        /**
         * Writes an auto property with a getter and (if the property is not ReadOnly) a setter.
         * This function can be used for both Classes and Interfaces.
         */
        writeAutoProperty(property, options) {
            if (!property)
                return;
            if (!options)
                options = {};
            this.writePropertyStart(property, options);
            this.write(property.isReadOnly ? ' {get;}' : ' {get; set;}');
            this.writeEndOfLine();
        }
        /**
         * Writes a property code block using optional callback functions for writing the getter and setter contents.
         */
        writePropertyBlock(property, getterContents, setterContents, options) {
            if (!property)
                return;
            if (options == null)
                options = {};
            this.writePropertyStart(property, options);
            this.writeEndOfLine();
            this.writeCodeBlock(() => {
                if (getterContents) {
                    this.writeLine('get');
                    this.writeCodeBlock(getterContents);
                }
                if (setterContents) {
                    this.writeLine('set');
                    this.writeCodeBlock(setterContents);
                }
            });
        }
        writePropertyStart(property, options) {
            const features = (options.features === undefined) ? exports.PropertyFeatures.All : options.features;
            const ownerIsInterface = elements.isInterface(property.owner);
            if (features & exports.PropertyFeatures.XmlDocSummary) {
                this.writeXmlDocSummary(property.ownedComments);
            }
            // Start a new, indented line        
            this.writeIndent();
            // Access modifier
            if (!ownerIsInterface && (features & exports.PropertyFeatures.AccessModifier)) {
                this.writeAccessModifier(property.visibility);
            }
            // Virtual
            if (options.virtual) {
                this.write('virtual ');
            }
            // Type
            var typeName = this.getTypeName(property) || 'object';
            if (property.isMultivalued()) {
                this.write(CSharpWriter.createCollectionType(typeName, options.collectionType));
            }
            else {
                this.write(typeName);
                if ((features & exports.PropertyFeatures.OptionalModifier) && property.lower === 0 && CSharpTypeNameProvider.canBeNullable(property.type)) {
                    this.write('?');
                }
            }
            // Name               
            this.write(` ${property.name}`);
        }
        /**
         * Gets a string containing the return type of the method. If the return type cannot be determined,
         * the string 'void' is returned. If the method returns a collection, a collection type is returned
         * based on the provided options.
         * @param operation The operation.
         * @param options An optional MethodOptions object.
         */
        getMethodReturnType(operation, options) {
            if (!operation)
                throw 'The operation is null or undefined.';
            const returnParameter = operation.getReturnParameter();
            if (!returnParameter)
                return 'void';
            const typeName = this.getTypeName(returnParameter);
            if (!typeName)
                return 'void'; // the type could not be mapped 
            if (!options)
                options = {};
            return operation.isMultivalued() ? CSharpWriter.createCollectionType(typeName, options.collectionType) : typeName;
        }
        /**
         * Writes the input and output parameters (all parameters except the return parameter) of a method.
         * @param params A collection of parameters.
         * @param options An optional MethodOptions object.
         */
        writeInOutParameters(params, options) {
            if (!params)
                return;
            if (!options)
                options = {};
            let i = 0;
            params.forEach((p) => {
                if (p.direction === elements.ParameterDirectionKind.return)
                    return;
                if (i > 0) {
                    this.write(', ');
                }
                if (p.direction === elements.ParameterDirectionKind.out) {
                    this.write('out ');
                }
                else if (p.direction === elements.ParameterDirectionKind.inout) {
                    this.write('ref '); // The ref keyword can be used for both value- and reference types
                }
                const typeName = this.getTypeName(p) || 'object';
                if (p.isMultivalued()) {
                    this.write(CSharpWriter.createCollectionType(typeName, options.collectionType));
                }
                else {
                    this.write(typeName);
                    if (p.lower === 0 && CSharpTypeNameProvider.canBeNullable(p.type)) {
                        this.write('?');
                    }
                }
                this.write(` ${p.name}`);
                i++;
            });
        }
        /**
        * Writes the visibility to the output with a trailing whitespace. If the visibilty is null or
        * not supported by C#, nothing will be written.
        * @param visibilityKind A VisibilityKind value. This value can be null.
        */
        writeAccessModifier(visibilityKind) {
            const accessModifierString = CSharpWriter.getAccessModifierString(visibilityKind);
            if (!accessModifierString)
                return;
            this.write(accessModifierString);
            this.writeWhiteSpace();
        }
        writeGeneralizations(generalizations, additional) {
            const allNames = [];
            if (generalizations) {
                // todo: allow qualifiedName and extend typeNameProvider with getGeneralTypeName?
                allNames.push(...generalizations.map(g => g.general.name));
            }
            if (additional) {
                allNames.push(...additional);
            }
            if (allNames.length === 0)
                return false;
            this.write(' : ');
            this.joinWrite(allNames, ', ', name => name);
            return true;
        }
        writeInterfaceRealizations(realizations, additional, hasGeneralizations) {
            const allNames = [];
            if (realizations) {
                // todo: allow qualifiedName and extend typeNameProvider with getInterfaceTypeName?
                allNames.push(...realizations.map(ir => ir.contract.name));
            }
            if (additional) {
                allNames.push(...additional);
            }
            if (allNames.length === 0)
                return false;
            this.write(hasGeneralizations ? ', ' : ' : ');
            this.joinWrite(allNames, ', ', name => name);
            return true;
        }
        getTypeName(typedElement) {
            if (!typedElement)
                return null;
            return this.typeNameProvider ? this.typeNameProvider.getTypeName(typedElement) : typedElement.getTypeName();
        }
        writeXmlDocSummary(data) {
            const lines = [];
            lines.push('<summary>');
            if (typeof data == 'string') {
                lines.push(data);
            }
            else {
                if (!data || data.length === 0)
                    return;
                lines.push(...data.map((c) => c.body));
            }
            lines.push('</summary>');
            this.commentWriter.writeCommentLines(lines, '/// ');
        }
        writeXmlDocParameters(element) {
            this.writeXmlDocLines(XmlDocUtility.getXmlDocLinesForParameters(element.getInputParameters()));
        }
        writeXmlDocReturns(element) {
            var returnParameter = element.getReturnParameter();
            if (!returnParameter)
                return;
            this.writeXmlDocLines([XmlDocUtility.getXmlDocLineForParameter(returnParameter)]);
        }
        writeXmlDocParagraph(text) {
            if (text == null)
                return;
            const lines = [text];
            this.commentWriter.writeCommentLines(lines, '/// ');
        }
        writeXmlDocLines(lines) {
            if (lines == null)
                return;
            this.commentWriter.writeCommentLines(lines, '/// ');
        }
        writeDelimitedCommentParagraph(text) {
            this.commentWriter.writeDelimitedCommentParagraph(text);
        }
        writeDelimitedCommentLines(lines) {
            this.commentWriter.writeDelimitedCommentLines(lines);
        }
        //#endregion Xml Docs    
        joinWrite(collection, separator, getStringFunc) {
            let isFirst = true;
            collection.forEach(c => {
                const value = getStringFunc(c);
                if (!value)
                    return;
                if (isFirst) {
                    isFirst = false;
                }
                else
                    this.write(separator);
                this.write(value);
            });
        }
        static getAccessModifierString(visibility) {
            switch (visibility) {
                case elements.VisibilityKind.public:
                    return 'public';
                case elements.VisibilityKind.private:
                    return 'private';
                case elements.VisibilityKind.protected:
                    return 'protected';
                case elements.VisibilityKind.package:
                    return 'internal';
                default:
                    return null;
            }
        }
        static createCollectionType(typeName, collectionType) {
            switch (collectionType) {
                case exports.CollectionType.IList:
                    return `IList<${typeName}>`;
                case exports.CollectionType.IEnumerable:
                    return `IEnumerable<${typeName}>`;
                default:
                    return `ICollection<${typeName}>`;
            }
        }
    }

    const reservedKeywords = ["abstract", "as", "base", "bool", "break", "byte", "case", "catch", "char", "checked", "class", "const", "continue", "decimal", "default", "delegate", "do", "double", "else",
        "enum", "event", "explicit", "extern", "false", "finally", "fixed", "float", "for", "foreach", "goto", "if", "implicit", "in", "int", "interface", "internal", "is", "lock", "long", "namespace", "new",
        "null", "object", "operator", "out", "override", "params", "private", "protected", "public", "readonly", "ref", "return", "sbyte", "sealed", "short", "sizeof", "stackalloc", "static", "string",
        "struct", "switch", "this", "throw", "true", "try", "typeof", "uint", "ulong", "unchecked", "unsafe", "ushort", "using", "virtual", "void", "volatile", "while"];
    /**
     * Prefixes reserved C# keywords with a "@". This applies to:
     * - Class names
     * - Interface names
     * - Enumeration names
     * - Attribute names
     * - Operation names
     * - Operation parameter names
     */
    class CSReservedKeywordTransform extends templating.RenamingTransform {
        constructor(targets = templating.RenameTargets.all) {
            super(targets);
        }
        rename(name, target) {
            if (reservedKeywords.indexOf(name) === -1)
                return name;
            return `@${name}`;
        }
    }

    exports.CSharpTypeNameProvider = CSharpTypeNameProvider;
    exports.CSharpWriter = CSharpWriter;
    exports.CSReservedKeywordTransform = CSReservedKeywordTransform;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
