import { TypeNameProvider } from '@yellicode/templating';
export interface WriterOptions {
    /**
     * The maximum width of generated documentation comments before they are word-wrapped.
     * The default value is 100 characters.
     */
    maxCommentWidth?: number;
    /**
     * Sets an optional TypeNameProvider. By default, the CSharpTypeNameProvider is used.
     */
    typeNameProvider?: TypeNameProvider;
}
/**
 * Enumerates the possible collection types to generated for properties and parameters.
 */
export declare enum CollectionType {
    ICollection = 0,
    IEnumerable = 1,
    IList = 2,
}
export declare enum NamespaceFeatures {
    None = 0,
    All = 0,
}
export interface NamespaceOptions {
    /**
     * Defines the namespace features to write. The default is NamespaceFeatures.All.
     */
    features?: NamespaceFeatures;
    /**
     * Indicates if the fully-qualified namespace name should be written instead of only the current one.
     * The full name will be created from the names of the Package and it's ancestor packages.
     * The default value is false.
     */
    writeFullName?: boolean;
}
export declare enum ClassFeatures {
    None = 0,
    XmlDocSummary = 1,
    Generalizations = 2,
    InterfaceRealizations = 4,
    All = 7,
    AllExceptXmlDocs = 6,
}
export interface ClassOptions {
    /**
     * Defines the class features to write. The default is ClassFeatures.All.
     */
    features?: ClassFeatures;
    /**
     * Indicates if the class must be prefixed with the "partial" keyword.
    */
    isPartial?: boolean;
    /**
     * Any additional interface names that the class should implement.
     */
    implements?: string[];
    /**
     * Any additional class names that the class should inherit from.
     */
    inherits?: string[];
}
export declare enum InterfaceFeatures {
    None = 0,
    XmlDocSummary = 1,
    Generalizations = 2,
    All = 3,
    AllExceptXmlDocs = 2,
}
export interface InterfaceOptions {
    /**
     * Defines the class features to write. The default is InterfaceFeatures.All.
     */
    features?: InterfaceFeatures;
    /**
      * Indicates if the interface must be prefixed with the "partial" keyword.
     */
    isPartial?: boolean;
    /**
    * Any additional interface names that the interface should inherit from.
    */
    inherits?: string[];
}
export declare enum EnumFeatures {
    None = 0,
    XmlDocSummary = 1,
    /**
     * Writes enumeration member initializers. This flag only applies when values are provided in the model.
     */
    Initializers = 2,
    All = 3,
    AllExceptXmlDocs = 2,
}
export declare enum EnumMemberFeatures {
    None = 0,
    XmlDocSummary = 1,
    /**
     * Writes enumeration member initializers. This flag only applies when values are provided in the model.
     */
    Initializers = 2,
    All = 3,
    AllExceptXmlDocs = 2,
}
export interface EnumOptions {
    features?: EnumFeatures;
}
export interface EnumMemberOptions {
    features?: EnumMemberFeatures;
}
export declare enum PropertyFeatures {
    None = 0,
    XmlDocSummary = 1,
    /**
     * The access modifier if the owner is not an Interface.
     */
    AccessModifier = 2,
    OptionalModifier = 4,
    All = 7,
    AllExceptXmlDocs = 6,
}
export interface PropertyOptions {
    /**
     * Defines the property features to write. The default is PropertyFeatures.All.
     */
    features?: PropertyFeatures;
    /**
    * Sets the collection type to be generated for the property in case it is multi-valued. The default value is ICollection.
    */
    collectionType?: CollectionType;
    /**
     * Indicates if the property should be made virtual. The default value is false.
     */
    virtual?: boolean;
}
export declare enum MethodFeatures {
    None = 0,
    XmlDocSummary = 1,
    XmlDocParameters = 2,
    XmlDocReturns = 4,
    /**
    * The access modifier if the owner is not an Interface.
    */
    AccessModifier = 8,
    All = 15,
    AllExceptXmlDocs = 8,
}
export interface MethodOptions {
    /**
     * Sets the MethodFeatures. The default is MethodFeatures.All.
     */
    features?: MethodFeatures;
    /**
     * Sets the collection type to be generated for parameters in case they are multi-valued. The default is ICollection.
     */
    collectionType?: CollectionType;
    /**
     * Indicates if the method should be made virtual. The default value is false.
     */
    virtual?: boolean;
}
